@@ -8,13 +8,30 @@
 from io import BytesIO
 import uuid
 
+import attr
 from django.urls import reverse
 import pytest
 from rest_framework import status
 
-from swh.deposit.config import COL_IRI, DEPOSIT_STATUS_DEPOSITED
+from swh.deposit.config import (
+    COL_IRI,
+    DEPOSIT_STATUS_DEPOSITED,
+    DEPOSIT_STATUS_LOAD_SUCCESS,
+    APIConfig,
+)
 from swh.deposit.models import Deposit, DepositCollection, DepositRequest
 from swh.deposit.parsers import parse_xml
+from swh.deposit.tests.common import post_atom
+from swh.deposit.utils import compute_metadata_context
+from swh.model.identifiers import SWHID, parse_swhid
+from swh.model.model import (
+    MetadataAuthority,
+    MetadataAuthorityType,
+    MetadataFetcher,
+    MetadataTargetType,
+    RawExtrinsicMetadata,
+)
+from swh.storage.interface import PagedResult
 
 
 def test_post_deposit_atom_201_even_with_decimal(
@@ -25,9 +42,9 @@ def test_post_deposit_atom_201_even_with_decimal(
     """
     atom_error_with_decimal = atom_dataset["error-with-decimal"]
 
-    response = authenticated_client.post(
+    response = post_atom(
+        authenticated_client,
         reverse(COL_IRI, args=[deposit_collection.name]),
-        content_type="application/atom+xml;type=entry",
         data=atom_error_with_decimal,
         HTTP_SLUG="external-id",
         HTTP_IN_PROGRESS="false",
@@ -54,9 +71,9 @@ def test_post_deposit_atom_400_with_empty_body(
 
     """
     atom_content = atom_dataset["entry-data-empty-body"]
-    response = authenticated_client.post(
+    response = post_atom(
+        authenticated_client,
         reverse(COL_IRI, args=[deposit_collection.name]),
-        content_type="application/atom+xml;type=entry",
         data=atom_content,
         HTTP_SLUG="external-id",
     )
@@ -70,9 +87,9 @@ def test_post_deposit_atom_400_badly_formatted_atom(
     """Posting a badly formatted atom should return a 400 response
 
     """
-    response = authenticated_client.post(
+    response = post_atom(
+        authenticated_client,
         reverse(COL_IRI, args=[deposit_collection.name]),
-        content_type="application/atom+xml;type=entry",
         data=atom_dataset["entry-data-badly-formatted"],
         HTTP_SLUG="external-id",
     )
@@ -86,9 +103,9 @@ def test_post_deposit_atom_parsing_error(
     """Posting parsing error prone atom should return 400
 
     """
-    response = authenticated_client.post(
+    response = post_atom(
+        authenticated_client,
         reverse(COL_IRI, args=[deposit_collection.name]),
-        content_type="application/atom+xml;type=entry",
         data=atom_dataset["entry-data-parsing-error-prone"],
         HTTP_SLUG="external-id",
     )
@@ -102,9 +119,9 @@ def test_post_deposit_atom_400_both_create_origin_and_add_to_origin(
     """Posting a badly formatted atom should return a 400 response
 
     """
-    response = authenticated_client.post(
+    response = post_atom(
+        authenticated_client,
         reverse(COL_IRI, args=[deposit_collection.name]),
-        content_type="application/atom+xml;type=entry",
         data=atom_dataset["entry-data-with-both-create-origin-and-add-to-origin"],
     )
     assert response.status_code == status.HTTP_400_BAD_REQUEST
@@ -114,32 +131,6 @@ def test_post_deposit_atom_400_both_create_origin_and_add_to_origin(
     ) in response.content
 
 
-def test_add_deposit_with_add_to_origin_and_external_identifier(
-    authenticated_client,
-    deposit_collection,
-    completed_deposit,
-    atom_dataset,
-    deposit_user,
-):
-    """Posting deposit with <swh:add_to_origin> creates a new deposit with parent
-
-    """
-    # given multiple deposit already loaded
-    origin_url = deposit_user.provider_url + completed_deposit.external_id
-
-    # adding a new deposit with the same external id as a completed deposit
-    # creates the parenting chain
-    response = authenticated_client.post(
-        reverse(COL_IRI, args=[deposit_collection.name]),
-        content_type="application/atom+xml;type=entry",
-        data=atom_dataset["entry-data-with-both-add-to-origin-and-external-id"]
-        % origin_url,
-    )
-
-    assert response.status_code == status.HTTP_400_BAD_REQUEST
-    assert b"&lt;external_identifier&gt; is deprecated." in response.content
-
-
 def test_post_deposit_atom_403_create_wrong_origin_url_prefix(
     authenticated_client, deposit_collection, atom_dataset, deposit_user
 ):
@@ -148,9 +139,9 @@ def test_post_deposit_atom_403_create_wrong_origin_url_prefix(
     """
     origin_url = "http://example.org/foo"
 
-    response = authenticated_client.post(
+    response = post_atom(
+        authenticated_client,
         reverse(COL_IRI, args=[deposit_collection.name]),
-        content_type="application/atom+xml;type=entry",
         data=atom_dataset["entry-data0"] % origin_url,
         HTTP_IN_PROGRESS="true",
     )
@@ -162,28 +153,6 @@ def test_post_deposit_atom_403_create_wrong_origin_url_prefix(
     assert expected_msg in response.content.decode()
 
 
-def test_post_deposit_atom_403_add_to_wrong_origin_url_prefix(
-    authenticated_client, deposit_collection, atom_dataset, deposit_user
-):
-    """Creating an origin for a prefix not owned by the client is forbidden
-
-    """
-    origin_url = "http://example.org/foo"
-
-    response = authenticated_client.post(
-        reverse(COL_IRI, args=[deposit_collection.name]),
-        content_type="application/atom+xml;type=entry",
-        data=atom_dataset["entry-data-with-add-to-origin"] % origin_url,
-        HTTP_IN_PROGRESS="true",
-    )
-    assert response.status_code == status.HTTP_403_FORBIDDEN
-    expected_msg = (
-        f"Cannot create origin {origin_url}, "
-        f"it must start with {deposit_user.provider_url}"
-    )
-    assert expected_msg in response.content.decode()
-
-
 def test_post_deposit_atom_use_slug_header(
     authenticated_client, deposit_collection, deposit_user, atom_dataset, mocker
 ):
@@ -196,9 +165,9 @@ def test_post_deposit_atom_use_slug_header(
     slug = str(uuid.uuid4())
 
     # when
-    response = authenticated_client.post(
+    response = post_atom(
+        authenticated_client,
         url,
-        content_type="application/atom+xml;type=entry",
         data=atom_dataset["entry-data-no-origin-url"],
         HTTP_IN_PROGRESS="false",
         HTTP_SLUG=slug,
@@ -226,11 +195,10 @@ def test_post_deposit_atom_no_origin_url_nor_slug_header(
     mocker.patch("uuid.uuid4", return_value=slug)
 
     # when
-    response = authenticated_client.post(
+    response = post_atom(
+        authenticated_client,
         url,
-        content_type="application/atom+xml;type=entry",
         data=atom_dataset["entry-data-no-origin-url"],
-        # + headers
         HTTP_IN_PROGRESS="false",
     )
 
@@ -255,11 +223,10 @@ def test_post_deposit_atom_with_mismatched_slug_and_external_identifier(
     url = reverse(COL_IRI, args=[deposit_collection.name])
 
     # when
-    response = authenticated_client.post(
+    response = post_atom(
+        authenticated_client,
         url,
-        content_type="application/atom+xml;type=entry",
         data=atom_dataset["error-with-external-identifier"] % external_id,
-        # + headers
         HTTP_IN_PROGRESS="false",
         HTTP_SLUG="something",
     )
@@ -284,12 +251,8 @@ def test_post_deposit_atom_with_create_origin_and_external_identifier(
     )
 
     # when
-    response = authenticated_client.post(
-        url,
-        content_type="application/atom+xml;type=entry",
-        data=document,
-        # + headers
-        HTTP_IN_PROGRESS="false",
+    response = post_atom(
+        authenticated_client, url, data=document, HTTP_IN_PROGRESS="false",
     )
 
     assert b"&lt;external_identifier&gt; is deprecated" in response.content
@@ -311,12 +274,8 @@ def test_post_deposit_atom_with_create_origin_and_reference(
     )
 
     # when
-    response = authenticated_client.post(
-        url,
-        content_type="application/atom+xml;type=entry",
-        data=document,
-        # + headers
-        HTTP_IN_PROGRESS="false",
+    response = post_atom(
+        authenticated_client, url, data=document, HTTP_IN_PROGRESS="false",
     )
 
     assert b"only one may be used on a given deposit" in response.content
@@ -331,9 +290,9 @@ def test_post_deposit_atom_unknown_collection(authenticated_client, atom_dataset
     with pytest.raises(DepositCollection.DoesNotExist):
         DepositCollection.objects.get(name=unknown_collection)
 
-    response = authenticated_client.post(
-        reverse(COL_IRI, args=[unknown_collection]),  # <- unknown collection
-        content_type="application/atom+xml;type=entry",
+    response = post_atom(
+        authenticated_client,
+        reverse(COL_IRI, args=[unknown_collection]),
         data=atom_dataset["entry-data0"],
         HTTP_SLUG="something",
     )
@@ -356,9 +315,9 @@ def test_post_deposit_atom_entry_initial(
     atom_entry_data = atom_dataset["entry-data0"] % origin_url
 
     # when
-    response = authenticated_client.post(
+    response = post_atom(
+        authenticated_client,
         reverse(COL_IRI, args=[deposit_collection.name]),
-        content_type="application/atom+xml;type=entry",
         data=atom_entry_data,
         HTTP_IN_PROGRESS="false",
     )
@@ -395,9 +354,9 @@ def test_post_deposit_atom_entry_with_codemeta(
 
     atom_entry_data = atom_dataset["codemeta-sample"] % origin_url
     # when
-    response = authenticated_client.post(
+    response = post_atom(
+        authenticated_client,
         reverse(COL_IRI, args=[deposit_collection.name]),
-        content_type="application/atom+xml;type=entry",
         data=atom_entry_data,
         HTTP_IN_PROGRESS="false",
     )
@@ -421,83 +380,249 @@ def test_post_deposit_atom_entry_with_codemeta(
     assert bool(deposit_request.archive) is False
 
 
-def test_post_deposit_atom_entry_multiple_steps(
-    authenticated_client, deposit_collection, atom_dataset, deposit_user
+def test_deposit_metadata_invalid(
+    authenticated_client, deposit_collection, atom_dataset
 ):
-    """After initial deposit, updating a deposit should return a 201
+    """Posting invalid swhid reference is bad request returned to client
 
     """
-    # given
-    origin_url = deposit_user.provider_url + "2225c695-cfb8-4ebb-aaaa-80da344efa6a"
+    invalid_swhid = "swh:1:dir :31b5c8cc985d190b5a7ef4878128ebfdc2358f49"
+    xml_data = atom_dataset["entry-data-with-swhid"].format(swhid=invalid_swhid)
 
-    with pytest.raises(Deposit.DoesNotExist):
-        deposit = Deposit.objects.get(origin_url=origin_url)
-
-    # when
-    response = authenticated_client.post(
+    response = post_atom(
+        authenticated_client,
         reverse(COL_IRI, args=[deposit_collection.name]),
-        content_type="application/atom+xml;type=entry",
-        data=atom_dataset["entry-data1"],
-        HTTP_IN_PROGRESS="True",
+        data=xml_data,
+    )
+    assert response.status_code == status.HTTP_400_BAD_REQUEST
+    assert b"Invalid SWHID reference" in response.content
+
+
+def test_deposit_metadata_fails_functional_checks(
+    authenticated_client, deposit_collection, atom_dataset
+):
+    """Posting functionally invalid metadata swhid is bad request returned to client
+
+    """
+    swhid = "swh:1:dir:31b5c8cc985d190b5a7ef4878128ebfdc2358f49"
+    invalid_xml_data = atom_dataset[
+        "entry-data-with-swhid-fail-metadata-functional-checks"
+    ].format(swhid=swhid)
+
+    response = post_atom(
+        authenticated_client,
+        reverse(COL_IRI, args=[deposit_collection.name]),
+        data=invalid_xml_data,
+    )
+    assert response.status_code == status.HTTP_400_BAD_REQUEST
+    assert b"Functional metadata checks failure" in response.content
+
+
+@pytest.mark.parametrize(
+    "swhid,target_type",
+    [
+        (
+            "swh:1:cnt:01b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+            MetadataTargetType.CONTENT,
+        ),
+        (
+            "swh:1:dir:11b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+            MetadataTargetType.DIRECTORY,
+        ),
+        (
+            "swh:1:rev:21b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+            MetadataTargetType.REVISION,
+        ),
+        (
+            "swh:1:rel:31b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+            MetadataTargetType.RELEASE,
+        ),
+        (
+            "swh:1:snp:41b5c8cc985d190b5a7ef4878128ebfdc2358f49",
+            MetadataTargetType.SNAPSHOT,
+        ),
+        (
+            "swh:1:cnt:51b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
+            MetadataTargetType.CONTENT,
+        ),
+        (
+            "swh:1:dir:c4993c872593e960dc84e4430dbbfbc34fd706d0;origin=https://inria.halpreprod.archives-ouvertes.fr/hal-01243573;visit=swh:1:snp:0175049fc45055a3824a1675ac06e3711619a55a;anchor=swh:1:rev:b5f505b005435fa5c4fa4c279792bd7b17167c04;path=/",  # noqa
+            MetadataTargetType.DIRECTORY,
+        ),
+        (
+            "swh:1:rev:71b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
+            MetadataTargetType.REVISION,
+        ),
+        (
+            "swh:1:rel:81b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
+            MetadataTargetType.RELEASE,
+        ),
+        (
+            "swh:1:snp:91b5c8cc985d190b5a7ef4878128ebfdc2358f49;origin=h://g.c/o/repo",
+            MetadataTargetType.SNAPSHOT,
+        ),
+    ],
+)
+def test_deposit_metadata_swhid(
+    swhid,
+    target_type,
+    authenticated_client,
+    deposit_collection,
+    atom_dataset,
+    swh_storage,
+):
+    """Posting a swhid reference is stored on raw extrinsic metadata storage
+
+    """
+    swhid_reference = parse_swhid(swhid)
+    swhid_core = attr.evolve(swhid_reference, metadata={})
+
+    xml_data = atom_dataset["entry-data-with-swhid"].format(swhid=swhid)
+    deposit_client = authenticated_client.deposit_client
+
+    response = post_atom(
+        authenticated_client,
+        reverse(COL_IRI, args=[deposit_collection.name]),
+        data=xml_data,
     )
 
-    # then
     assert response.status_code == status.HTTP_201_CREATED
-
     response_content = parse_xml(BytesIO(response.content))
+
+    # Ensure the deposit is finalized
     deposit_id = int(response_content["swh:deposit_id"])
-
     deposit = Deposit.objects.get(pk=deposit_id)
-    assert deposit.collection == deposit_collection
-    assert deposit.origin_url is None  # not provided yet
-    assert deposit.status == "partial"
+    assert isinstance(swhid_core, SWHID)
+    assert deposit.swhid == str(swhid_core)
+    assert deposit.swhid_context == str(swhid_reference)
+    assert deposit.complete_date == deposit.reception_date
+    assert deposit.complete_date is not None
+    assert deposit.status == DEPOSIT_STATUS_LOAD_SUCCESS
 
-    # one associated request to a deposit
-    deposit_requests = DepositRequest.objects.filter(deposit=deposit)
-    assert len(deposit_requests) == 1
-
-    atom_entry_data = atom_dataset["entry-only-create-origin"] % (origin_url)
-
-    for link in response_content["atom:link"]:
-        if link["@rel"] == "http://purl.org/net/sword/terms/add":
-            se_iri = link["@href"]
-            break
-    else:
-        assert False, f"missing SE-IRI from {response_content['link']}"
-
-    # when updating the first deposit post
-    response = authenticated_client.post(
-        se_iri,
-        content_type="application/atom+xml;type=entry",
-        data=atom_entry_data,
-        HTTP_IN_PROGRESS="False",
+    # Ensure metadata stored in the metadata storage is consistent
+    metadata_authority = MetadataAuthority(
+        type=MetadataAuthorityType.DEPOSIT_CLIENT,
+        url=deposit_client.provider_url,
+        metadata={"name": deposit_client.last_name},
     )
 
-    # then
-    assert response.status_code == status.HTTP_201_CREATED, response.content.decode()
+    actual_authority = swh_storage.metadata_authority_get(
+        MetadataAuthorityType.DEPOSIT_CLIENT, url=deposit_client.provider_url
+    )
+    assert actual_authority == metadata_authority
 
+    config = APIConfig()
+    metadata_fetcher = MetadataFetcher(
+        name=config.tool["name"],
+        version=config.tool["version"],
+        metadata=config.tool["configuration"],
+    )
+
+    actual_fetcher = swh_storage.metadata_fetcher_get(
+        config.tool["name"], config.tool["version"]
+    )
+    assert actual_fetcher == metadata_fetcher
+
+    page_results = swh_storage.raw_extrinsic_metadata_get(
+        target_type, swhid_core, metadata_authority
+    )
+    discovery_date = page_results.results[0].discovery_date
+
+    assert len(page_results.results) == 1
+    assert page_results.next_page_token is None
+
+    object_type, metadata_context = compute_metadata_context(swhid_reference)
+    assert page_results == PagedResult(
+        results=[
+            RawExtrinsicMetadata(
+                type=object_type,
+                target=swhid_core,
+                discovery_date=discovery_date,
+                authority=attr.evolve(metadata_authority, metadata=None),
+                fetcher=attr.evolve(metadata_fetcher, metadata=None),
+                format="sword-v2-atom-codemeta",
+                metadata=xml_data.encode(),
+                **metadata_context,
+            )
+        ],
+        next_page_token=None,
+    )
+    assert deposit.complete_date == discovery_date
+
+
+@pytest.mark.parametrize(
+    "url", ["https://gitlab.org/user/repo", "https://whatever.else/repo",]
+)
+def test_deposit_metadata_origin(
+    url, authenticated_client, deposit_collection, atom_dataset, swh_storage,
+):
+    """Posting a swhid reference is stored on raw extrinsic metadata storage
+
+    """
+    xml_data = atom_dataset["entry-data-with-origin-reference"].format(url=url)
+    deposit_client = authenticated_client.deposit_client
+    response = post_atom(
+        authenticated_client,
+        reverse(COL_IRI, args=[deposit_collection.name]),
+        data=xml_data,
+    )
+
+    assert response.status_code == status.HTTP_201_CREATED
     response_content = parse_xml(BytesIO(response.content))
+    # Ensure the deposit is finalized
     deposit_id = int(response_content["swh:deposit_id"])
-
     deposit = Deposit.objects.get(pk=deposit_id)
-    assert deposit.collection == deposit_collection
-    assert deposit.origin_url == origin_url
-    assert deposit.status == DEPOSIT_STATUS_DEPOSITED
+    # we got not swhid as input so we cannot have those
+    assert deposit.swhid is None
+    assert deposit.swhid_context is None
+    assert deposit.complete_date == deposit.reception_date
+    assert deposit.complete_date is not None
+    assert deposit.status == DEPOSIT_STATUS_LOAD_SUCCESS
 
-    assert len(Deposit.objects.all()) == 1
+    # Ensure metadata stored in the metadata storage is consistent
+    metadata_authority = MetadataAuthority(
+        type=MetadataAuthorityType.DEPOSIT_CLIENT,
+        url=deposit_client.provider_url,
+        metadata={"name": deposit_client.last_name},
+    )
 
-    # now 2 associated requests to a same deposit
-    deposit_requests = DepositRequest.objects.filter(deposit=deposit).order_by("id")
-    assert len(deposit_requests) == 2
+    actual_authority = swh_storage.metadata_authority_get(
+        MetadataAuthorityType.DEPOSIT_CLIENT, url=deposit_client.provider_url
+    )
+    assert actual_authority == metadata_authority
 
-    atom_entry_data1 = atom_dataset["entry-data1"]
-    expected_meta = [
-        {"metadata": parse_xml(atom_entry_data1), "raw_metadata": atom_entry_data1},
-        {"metadata": parse_xml(atom_entry_data), "raw_metadata": atom_entry_data},
-    ]
+    config = APIConfig()
+    metadata_fetcher = MetadataFetcher(
+        name=config.tool["name"],
+        version=config.tool["version"],
+        metadata=config.tool["configuration"],
+    )
 
-    for i, deposit_request in enumerate(deposit_requests):
-        actual_metadata = deposit_request.metadata
-        assert actual_metadata == expected_meta[i]["metadata"]
-        assert deposit_request.raw_metadata == expected_meta[i]["raw_metadata"]
-        assert bool(deposit_request.archive) is False
+    actual_fetcher = swh_storage.metadata_fetcher_get(
+        config.tool["name"], config.tool["version"]
+    )
+    assert actual_fetcher == metadata_fetcher
+
+    page_results = swh_storage.raw_extrinsic_metadata_get(
+        MetadataTargetType.ORIGIN, url, metadata_authority
+    )
+    discovery_date = page_results.results[0].discovery_date
+
+    assert len(page_results.results) == 1
+    assert page_results.next_page_token is None
+
+    assert page_results == PagedResult(
+        results=[
+            RawExtrinsicMetadata(
+                type=MetadataTargetType.ORIGIN,
+                target=url,
+                discovery_date=discovery_date,
+                authority=attr.evolve(metadata_authority, metadata=None),
+                fetcher=attr.evolve(metadata_fetcher, metadata=None),
+                format="sword-v2-atom-codemeta",
+                metadata=xml_data.encode(),
+            )
+        ],
+        next_page_token=None,
+    )
+    assert deposit.complete_date == discovery_date
